@@ -1,21 +1,21 @@
 /*
- * This file is part of oFono - Open Source Telephony
  *
- * Copyright (C) 2009-2010  Nokia Corporation and/or its subsidiary(-ies).
+ *  oFono - Open Source Telephony
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * version 2 as published by the Free Software Foundation.
+ *  Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License version 2 as
+ *  published by the Free Software Foundation.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
- * 02110-1301 USA
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -52,6 +52,7 @@ struct _GIsiRequest {
 	guint timeout;
 	GIsiResponseFunc func;
 	void *data;
+	GDestroyNotify notify;
 };
 
 struct _GIsiIndication {
@@ -63,6 +64,7 @@ typedef struct _GIsiIndication GIsiIndication;
 
 struct _GIsiClient {
 	uint8_t resource;
+	uint16_t server_obj;
 	struct {
 		int major;
 		int minor;
@@ -200,6 +202,31 @@ int g_isi_version_minor(GIsiClient *client)
 	return client ? client->version.minor : -1;
 }
 
+/**
+ * Set the server object for the resource associated with @a
+ * client.
+ * @param client client for the resource
+ * @param server object
+ */
+void g_isi_server_object_set(GIsiClient *client, uint16_t obj)
+{
+	if (!client)
+		return;
+
+	client->server_obj = obj;
+}
+
+/**
+ * Returns the server object for the the resource associated with @a
+ * client.
+ * @param client client for the resource
+ * @return server object
+ */
+uint8_t g_isi_server_object(GIsiClient *client)
+{
+	return client ? client->server_obj : 0;
+}
+
 /**
  * Returns the resource associated with @a client
  * @param client client for the resource
@@ -240,6 +267,9 @@ static void g_isi_cleanup_req(void *data)
 		req->func(req->client, NULL, 0, 0, req->data);
 	req->client->error = 0;
 
+	if (req->notify)
+		req->notify(req->data);
+
 	if (req->timeout > 0)
 		g_source_remove(req->timeout);
 
@@ -256,43 +286,6 @@ static void g_isi_cleanup_ind(void *data)
 	g_free(ind);
 }
 
-static int g_isi_indication_init(GIsiClient *client)
-{
-	GIOChannel *channel;
-	uint8_t msg[] = {
-		0, PNS_SUBSCRIBED_RESOURCES_IND,
-		1, client->resource,
-	};
-
-	channel = phonet_new(client->modem, PN_COMMGR);
-	if (!channel)
-		return errno;
-
-	client->inds.fd = g_io_channel_unix_get_fd(channel);
-
-	/* Subscribe by sending an indication */
-	sendto(client->inds.fd, msg, 4, MSG_NOSIGNAL, (void *)&commgr,
-		sizeof(commgr));
-	client->inds.source = g_io_add_watch(channel,
-					G_IO_IN|G_IO_ERR|G_IO_HUP|G_IO_NVAL,
-					g_isi_callback, client);
-
-	g_io_channel_unref(channel);
-	return 0;
-}
-
-static void g_isi_indication_deinit(GIsiClient *client)
-{
-	uint8_t msg[] = {
-		0, PNS_SUBSCRIBED_RESOURCES_IND,
-		0,
-	};
-
-	/* Unsubscribe by sending an empty subscribe indication */
-	sendto(client->inds.fd, msg, 3, MSG_NOSIGNAL, (void *)&commgr,
-		sizeof(commgr));
-}
-
 /**
  * Destroys an ISI client, cancels all pending transactions and subscriptions.
  * @param client client to destroy (may be NULL)
@@ -303,15 +296,16 @@ void g_isi_client_destroy(GIsiClient *client)
 		return;
 
 	tdestroy(client->reqs.pending, g_isi_cleanup_req);
-	tdestroy(client->inds.subs, g_isi_cleanup_ind);
-
 	if (client->reqs.source > 0)
 		g_source_remove(client->reqs.source);
 
+	tdestroy(client->inds.subs, g_isi_cleanup_ind);
+	client->inds.subs = NULL;
+	client->inds.count = 0;
+	g_isi_commit_subscriptions(client);
 	if (client->inds.source > 0)
 		g_source_remove(client->inds.source);
 
-	g_isi_indication_deinit(client);
 	g_free(client);
 }
 
@@ -329,15 +323,7 @@ GIsiRequest *g_isi_request_make(GIsiClient *client, const void *__restrict buf,
 				size_t len, unsigned timeout,
 				GIsiResponseFunc cb, void *opaque)
 {
-	const struct iovec iov = {
-		.iov_base = (void *)buf,
-		.iov_len = len,
-	};
-
-	if (!client)
-		return NULL;
-
-	return g_isi_request_vmake(client, &iov, 1, timeout, cb, opaque);
+	return g_isi_send(client, buf, len, timeout, cb, opaque, NULL);
 }
 
 /**
@@ -350,18 +336,114 @@ GIsiRequest *g_isi_request_make(GIsiClient *client, const void *__restrict buf,
  * @param cb callback to process response(s)
  * @param opaque data for the callback
  */
-GIsiRequest *g_isi_request_vmake(GIsiClient *client,
-					const struct iovec *__restrict iov,
+GIsiRequest *g_isi_request_vmake(GIsiClient *client, const struct iovec *iov,
 					size_t iovlen, unsigned timeout,
-					GIsiResponseFunc cb, void *opaque)
+					GIsiResponseFunc func, void *opaque)
+{
+	return g_isi_vsend(client, iov, iovlen, timeout, func, opaque, NULL);
+}
+
+/**
+ * Send an ISI request to a specific Phonet address and register a callback
+ * to process the response(s) to the resulting transaction.
+ *
+ * @param client ISI client (from g_isi_client_create())
+ * @param dst Phonet destination address
+ * @param buf pointer to request payload
+ * @param len request payload byte length
+ * @param timeout timeout in seconds
+ * @param cb callback to process response(s)
+ * @param opaque data for the callback
+ * @param notify finalizer function for the @a opaque data (may be NULL)
+ *
+ * @return
+ * A pointer to a newly created GIsiRequest.
+ *
+ * @errors
+ * If an error occurs, @a errno is set accordingly and a NULL pointer is
+ * returned.
+ */
+GIsiRequest *g_isi_sendto(GIsiClient *client,
+				struct sockaddr_pn *dst,
+				const void *__restrict buf, size_t len,
+				unsigned timeout,
+				GIsiResponseFunc cb, void *opaque,
+				GDestroyNotify notify)
+{
+	const struct iovec iov = {
+		.iov_base = (void *)buf,
+		.iov_len = len,
+	};
+
+	return g_isi_vsendto(client, dst, &iov, 1, timeout, cb, opaque, notify);
+}
+
+
+/**
+ * Send an ISI request and register a callback to process the response(s) to
+ * the resulting transaction.
+ *
+ * @param cl ISI client (from g_isi_client_create())
+ * @param buf pointer to request payload
+ * @param len request payload byte length
+ * @param timeout timeout in seconds
+ * @param cb callback to process response(s)
+ * @param opaque data for the callback
+ * @param notify finalizer function for the @a opaque data (may be NULL)
+ *
+ * @return
+ * A pointer to a newly created GIsiRequest.
+ *
+ * @errors
+ * If an error occurs, @a errno is set accordingly and a NULL pointer is
+ * returned.
+ */
+GIsiRequest *g_isi_send(GIsiClient *client,
+			const void *__restrict buf, size_t len,
+			unsigned timeout,
+			GIsiResponseFunc cb, void *opaque,
+			GDestroyNotify notify)
+{
+	const struct iovec iov = {
+		.iov_base = (void *)buf,
+		.iov_len = len,
+	};
+
+	return g_isi_vsend(client, &iov, 1, timeout, cb, opaque, notify);
+}
+
+
+/**
+ * Send an ISI request to a specific Phonet address and register a callback
+ * to process the response(s) to the resulting transaction.
+ *
+ * @param client ISI client (from g_isi_client_create())
+ * @param dst Phonet destination address
+ * @param iov scatter-gather array to the request payload
+ * @param iovlen number of vectors in the scatter-gather array
+ * @param timeout timeout in seconds
+ * @param cb callback to process response(s)
+ * @param opaque data for the callback
+ * @param notify finalizer function for the @a opaque data (may be NULL)
+ *
+ * @return
+ * A pointer to a newly created GIsiRequest.
+ *
+ * @errors
+ * If an error occurs, @a errno is set accordingly and a NULL pointer is
+ * returned.
+ */
+GIsiRequest *g_isi_vsendto(GIsiClient *client,
+				struct sockaddr_pn *dst,
+				const struct iovec *__restrict iov,
+				size_t iovlen, unsigned timeout,
+				GIsiResponseFunc cb, void *opaque,
+				GDestroyNotify notify)
 {
 	struct iovec _iov[1 + iovlen];
-	struct sockaddr_pn dst = {
-		.spn_family = AF_PHONET,
-	};
 	struct msghdr msg = {
-		.msg_name = (void *)&dst,
-		.msg_namelen = sizeof(dst),
+		.msg_name = (void *)dst,
+		.msg_namelen = sizeof(*dst),
 		.msg_iov = _iov,
 		.msg_iovlen = 1 + iovlen,
 		.msg_control = NULL,
@@ -370,9 +452,10 @@ GIsiRequest *g_isi_request_vmake(GIsiClient *client,
 	};
 	ssize_t ret;
 	size_t i, len;
+	unsigned int key;
 	uint8_t id;
 
-	GIsiRequest *req;
+	GIsiRequest *req = NULL;
 	GIsiRequest **old;
 
 	if (!client) {
@@ -380,24 +463,33 @@ GIsiRequest *g_isi_request_vmake(GIsiClient *client,
 		return NULL;
 	}
 
-	req = g_try_new0(GIsiRequest, 1);
-	if (!req) {
-		errno = ENOMEM;
-		return NULL;
-	}
+	key = 1 + ((client->reqs.last + 1) % 255);
 
-	req->client = client;
-	req->id = (client->reqs.last + 1) % 255;
-	req->func = cb;
-	req->data = opaque;
+	if (cb) {
+		req = g_try_new0(GIsiRequest, 1);
+		if (!req) {
+			errno = ENOMEM;
+			return NULL;
+		}
 
-	old = tsearch(req, &client->reqs.pending, g_isi_cmp);
-	if (!old) {
-		errno = ENOMEM;
-		goto error;
-	}
+		req->client = client;
+		req->id = key;
+		req->func = cb;
+		req->data = opaque;
+		req->notify = notify;
 
-	if (*old != req) {
+		old = tsearch(req, &client->reqs.pending, g_isi_cmp);
+		if (!old) {
+			errno = ENOMEM;
+			goto error;
+		}
+		if (*old == req)
+			old = NULL;
+
+	} else
+		old = tfind(&key, &client->reqs.pending, g_isi_cmp);
+
+	if (old) {
 		/* FIXME: perhaps retry with randomized access after
 		 * initial miss. Although if the rate at which
 		 * requests are sent is so high that the transaction
@@ -407,9 +499,7 @@ GIsiRequest *g_isi_request_vmake(GIsiClient *client,
 		goto error;
 	}
 
-	dst.spn_resource = client->resource,
-
-	id = req->id;
+	id = key;
 	_iov[0].iov_base = &id;
 	_iov[0].iov_len = 1;
 
@@ -431,16 +521,59 @@ GIsiRequest *g_isi_request_vmake(GIsiClient *client,
 		goto error;
 	}
 
-	req->timeout = g_timeout_add_seconds(timeout, g_isi_timeout, req);
-	client->reqs.last = req->id;
+	if (req && timeout)
+		req->timeout = g_timeout_add_seconds(timeout, g_isi_timeout,
+							req);
+	client->reqs.last = key;
 	return req;
 
 error:
 	tdelete(req, &client->reqs.pending, g_isi_cmp);
 	g_free(req);
+
 	return NULL;
 }
 
+/**
+ * Send an ISI request and register a callback to process the response(s) to
+ * the resulting transaction.
+ *
+ * @param cl ISI client (from g_isi_client_create())
+ * @param iov scatter-gather array to the request payload
+ * @param iovlen number of vectors in the scatter-gather array
+ * @param timeout timeout in seconds
+ * @param cb callback to process response(s)
+ * @param opaque data for the callback
+ * @param notify finalizer function for the @a opaque data (may be NULL)
+ *
+ * @return
+ * A pointer to a newly created GIsiRequest.
+ *
+ * @errors
+ * If an error occurs, @a errno is set accordingly and a NULL pointer is
+ * returned.
+ */
+GIsiRequest *g_isi_vsend(GIsiClient *client,
+				const struct iovec *__restrict iov,
+				size_t iovlen, unsigned timeout,
+				GIsiResponseFunc cb, void *opaque,
+				GDestroyNotify notify)
+{
+	struct sockaddr_pn dst = {
+		.spn_family = AF_PHONET,
+	};
+
+	if (!client) {
+		errno = EINVAL;
+		return NULL;
+	}
+
+	dst.spn_resource = client->resource;
+
+	return g_isi_vsendto(client, &dst, iov, iovlen, timeout,
+				cb, opaque, notify);
+}
+
 /**
  * Cancels a pending request, i.e. stop waiting for responses and cancels the
  * timeout.
@@ -455,33 +588,103 @@ void g_isi_request_cancel(GIsiRequest *req)
 		g_source_remove(req->timeout);
 
 	tdelete(req, &req->client->reqs.pending, g_isi_cmp);
+
+	if (req->notify)
+		req->notify(req->data);
+
 	g_free(req);
 }
 
+static uint8_t *__msg;
+static void build_subscribe_msg(const void *nodep,
+				const VISIT which,
+				const int depth)
+{
+	GIsiIndication *ind = *(GIsiIndication **)nodep;
+	uint8_t res = ind->type >> 8;
+
+	switch (which) {
+	case postorder:
+	case leaf:
+		if (__msg[2] && res == __msg[2+__msg[2]])
+			break;
+		__msg[2]++;
+		__msg[2+__msg[2]] = res;
+		break;
+	default:
+		break;
+	}
+}
+
 /**
- * Subscribe to a given indication type for the resource that an ISI client
- * is associated with. If the same type was already subscribed, the old
- * subscription is overriden.
- * @param cl ISI client (fomr g_isi_client_create())
+ * Subscribe indications from the modem.
+ * @param client ISI client (from g_isi_client_create())
+ * @return 0 on success, a system error code otherwise.
+ */
+int g_isi_commit_subscriptions(GIsiClient *client)
+{
+	GIOChannel *channel;
+	uint8_t msg[3+256] = {
+		0, PNS_SUBSCRIBED_RESOURCES_IND,
+		0,
+	};
+
+	if (!client)
+		return -EINVAL;
+
+	if (!client->inds.source) {
+		if (client->inds.count == 0)
+			return 0;
+
+		channel = phonet_new(client->modem, PN_COMMGR);
+		if (!channel)
+			return -errno;
+
+		client->inds.fd = g_io_channel_unix_get_fd(channel);
+
+		client->inds.source = g_io_add_watch(channel,
+						G_IO_IN|G_IO_ERR|
+						G_IO_HUP|G_IO_NVAL,
+						g_isi_callback, client);
+
+		g_io_channel_unref(channel);
+	}
+
+	__msg = msg;
+	twalk(client->inds.subs, build_subscribe_msg);
+
+	/* Subscribe by sending an indication */
+	sendto(client->inds.fd, msg, 3+msg[2], MSG_NOSIGNAL, (void *)&commgr,
+		sizeof(commgr));
+	return 0;
+}
+
+/**
+ * Add subscription for a given indication type from the given resource.
+ * If the same type was already subscribed, the old subscription
+ * is overriden. Subscriptions for newly added resources do not become
+ * effective until g_isi_commit_subscriptions() has been called.
+ * @param client ISI client (from g_isi_client_create())
+ * @param res resource id
  * @param type indication type
  * @param cb callback to process received indications
  * @param data data for the callback
  * @return 0 on success, a system error code otherwise.
  */
-int g_isi_subscribe(GIsiClient *client, uint8_t type,
-			GIsiIndicationFunc cb, void *data)
+int g_isi_add_subscription(GIsiClient *client, uint8_t res, uint8_t type,
+				GIsiIndicationFunc cb, void *data)
 {
 	GIsiIndication *ind;
 	GIsiIndication **old;
 
-	if (cb == NULL)
+	if (client == NULL || cb == NULL)
 		return -EINVAL;
 
 	ind = g_try_new0(GIsiIndication, 1);
 	if (!ind)
 		return -ENOMEM;
 
-	ind->type = type;
+	ind->type = (res << 8) | type;
 
 	old = tsearch(ind, &client->inds.subs, g_isi_cmp);
 	if (!old) {
@@ -495,32 +698,54 @@ int g_isi_subscribe(GIsiClient *client, uint8_t type,
 	if (*old != ind) {
 		g_free(ind);
 		ind = *old;
-	}
+	} else
+		client->inds.count++;
 
 	ind->func = cb;
 	ind->data = data;
 
-	if (client->inds.count == 0) {
-		int ret = g_isi_indication_init(client);
-		if (ret) {
-			tdelete(ind, &client->inds.subs, g_isi_cmp);
-			g_free(ind);
-			return ret;
-		}
-		client->inds.count++;
-	}
 	return 0;
 }
 
 /**
- * Unsubscribe from a given indication type.
- * @param client ISI client (from g_isi_client_create())
- * @param type indication type.
+ * Subscribe to a given indication type for the resource that an ISI client
+ * is associated with. If the same type was already subscribed, the old
+ * subscription is overriden. For multiple subscriptions,
+ * g_isi_add_subcription() and g_isi_commit_subscriptions() should be used
+ * instead.
+ * @param cl ISI client (from g_isi_client_create())
+ * @param type indication type
+ * @param cb callback to process received indications
+ * @param data data for the callback
+ * @return 0 on success, a system error code otherwise.
  */
-void g_isi_unsubscribe(GIsiClient *client, uint8_t type)
+int g_isi_subscribe(GIsiClient *client, uint8_t type,
+			GIsiIndicationFunc cb, void *data)
+{
+	int ret;
+
+	if (!client)
+		return -EINVAL;
+
+	ret = g_isi_add_subscription(client, client->resource, type, cb, data);
+	if (ret)
+		return ret;
+
+	return g_isi_commit_subscriptions(client);
+}
+
+/**
+ * Remove subscription for a given indication type from the given resource.
+ * g_isi_commit_subcsriptions() should be called after modifications to
+ * cancel unnecessary resource subscriptions from the modem.
+ * @param client ISI client (from g_isi_client_create())
+ * @param res resource id
+ * @param type indication type
+ */
+void g_isi_remove_subscription(GIsiClient *client, uint8_t res, uint8_t type)
 {
 	GIsiIndication *ind;
-	unsigned int id = type;
+	unsigned int id = (res << 8) | type;
 
 	if (!client)
 		return;
@@ -529,18 +754,33 @@ void g_isi_unsubscribe(GIsiClient *client, uint8_t type)
 	if (!ind)
 		return;
 
-	if (--client->inds.count == 0)
-		g_isi_indication_deinit(client);
-
+	client->inds.count--;
 	g_free(ind);
 }
 
-static void g_isi_dispatch_indication(GIsiClient *client, uint16_t obj,
-					uint8_t *msg, size_t len)
+/**
+ * Unsubscribe from a given indication type. For removing multiple
+ * subscriptions, g_isi_remove_subcription() and
+ * g_isi_commit_subscriptions() should be used instead.
+ * @param client ISI client (from g_isi_client_create())
+ * @param type indication type.
+ */
+void g_isi_unsubscribe(GIsiClient *client, uint8_t type)
+{
+	if (!client)
+		return;
+
+	g_isi_remove_subscription(client, client->resource, type);
+	g_isi_commit_subscriptions(client);
+}
+
+static void g_isi_dispatch_indication(GIsiClient *client, uint8_t res,
+					uint16_t obj, uint8_t *msg,
+					size_t len)
 {
 	void *ret;
 	GIsiIndication *ind;
-	unsigned type = msg[0];
+	unsigned type = (res << 8) | msg[0];
 
 	ret = tfind(&type, &client->inds.subs, g_isi_cmp);
 	if (!ret)
@@ -552,8 +792,9 @@ static void g_isi_dispatch_indication(GIsiClient *client, uint16_t obj,
 		ind->func(client, msg, len, obj, ind->data);
 }
 
-static void g_isi_dispatch_response(GIsiClient *client, uint16_t obj,
-					uint8_t *msg, size_t len)
+static void g_isi_dispatch_response(GIsiClient *client, uint8_t res,
+					uint16_t obj, uint8_t *msg,
+					size_t len)
 {
 	void *ret;
 	GIsiRequest *req;
@@ -564,7 +805,7 @@ static void g_isi_dispatch_response(GIsiClient *client, uint16_t obj,
 		/* This could either be an unsolicited response, which
 		 * we will ignore, or an incoming request, which we
 		 * handle just like an incoming indication */
-		g_isi_dispatch_indication(client, obj, msg + 1, len - 1);
+		g_isi_dispatch_indication(client, res, obj, msg + 1, len - 1);
 		return;
 	}
 
@@ -596,7 +837,7 @@ static gboolean g_isi_callback(GIOChannel *channel, GIOCondition cond,
 		uint8_t res;
 
 		len = phonet_read(channel, buf, len, &obj, &res);
-		if (len < 2 || res != client->resource)
+		if (len < 2)
 			return TRUE;
 
 		msg = (uint8_t *)buf;
@@ -606,11 +847,11 @@ static gboolean g_isi_callback(GIOChannel *channel, GIOCondition cond,
 						client->debug_data);
 
 		if (fd == client->reqs.fd)
-			g_isi_dispatch_response(client, obj, msg, len);
+			g_isi_dispatch_response(client, res, obj, msg, len);
 		else
 			/* Transaction field at first byte is
 			 * discarded with indications */
-			g_isi_dispatch_indication(client, obj, msg + 1,
+			g_isi_dispatch_indication(client, res, obj, msg + 1,
 							len - 1);
 	}
 	return TRUE;
